# convert Agilent Data to IsoMS format
# usage:
#      AgilentDataConvert<-function(peaklistDataFolder,dataSaveFolder,nLineSkip,maxPeaksInLine,isNoSampleNO)
#            peaklistDataFolder: Agilent csv file folder, default is "./"
#        dataSaveFolder: folder to save the converted results, default is "./converted/"
#             nLineSkip: number of lines to skip (header information) in Agilent csv file, default is 10
#        maxPeaksInLine: max number of peaks in each line of Agilent csv file, used to preallocated the output matrix, default is 2000
#          isNoSampleNO: sample No. is NOT available in Agilent csv file, default is 1.
# e.g.,
#      source("AgilentDataConvert.R")
#      inP<-"./"
#      outP<-"./converted/"
#      AgilentDataConvert(inP,outP);

AgilentDataConvert<-function(peaklistDataFolder="./",convertedDataFolder="./input/",nLineSkip=10,maxPeaksInLine=5000,isNoSampleNO=1)
{
  # create the saving folder if necessary
  if ( ! file.exists(convertedDataFolder) )
  {
     dir.create(convertedDataFolder,showWarnings=FALSE, recursive=TRUE)
  }

  # list all csv file
  csvfiles<-list.files(path=peaklistDataFolder,pattern="(.csv)$");
  for ( nf in 1:length(csvfiles) ) 
  {
      # read data from each csv file
      oriCSVfile<-csvfiles[nf]
      saveCSVfile<-paste(convertedDataFolder,"forIsoMS_",oriCSVfile,sep='')

      print("",quote = FALSE);
      print(paste(oriCSVfile, "was read at:", format(Sys.time(),"%Y-%B-%d %H:%M:%S")),quote = FALSE)

      # read as characters
      inputcsv<-paste(peaklistDataFolder,oriCSVfile,sep='')
      allLines<-readLines(inputcsv);
      dbLines<-allLines[(nLineSkip+1):length(allLines)]; rm(allLines)
      dbLines<-gsub("peak","",dbLines)
      dbLines<-gsub("\\+","",dbLines)

      # create a temporary matrix [with big enough columns]
      nLine<-length(dbLines)
      dat<-matrix(nrow=nLine,ncol=2*maxPeaksInLine+7)
      for (nl in 1:nLine)
      {
           numberInCurrentLine<-scan(text=dbLines[nl],sep=",",nlines=1,fill=FALSE,flush=FALSE,quiet=TRUE)
           dat[nl,1:length(numberInCurrentLine)]<-numberInCurrentLine  
      }

      #find valid lines
      indValid<-which(!is.na(dat[,7]) & dat[,7]>=1)
      numPeak<-sum(dat[indValid,7])
      
      #create the export matrix
      pkmat<-matrix(nrow=numPeak, ncol=5)
      colnames(pkmat)<-c("sample","No","rt","mz", "maxo")
      ind0 <- 1
      for (ns in 1:length(indValid))
      {
          indRow<-indValid[ns]
          numPKc<-dat[indRow,7]
          indCol<-2*numPKc+7
          RTc<-dat[indRow,1]          # retention time
          ScanNOc<-ns                 # scan number [arbitrary]
          if (isNoSampleNO==0) {SAMPLEc<-dat[indRow,2]}

          ind1<-ind0+numPKc-1
          if (isNoSampleNO==0) {pkmat[ind0:ind1,1]<-SAMPLEc}
          pkmat[ind0:ind1,2]<-ScanNOc # rethink how to define scan number  
          pkmat[ind0:ind1,3]<-RTc*60
          pkmat[ind0:ind1,4]<-dat[indRow,seq(8,indCol,2)]
          pkmat[ind0:ind1,5]<-dat[indRow,seq(9,indCol,2)]
          ind0<-ind1+1
      } 
      if (isNoSampleNO==1) {pkmat[,1]<- 1; }
      # export the matrix
      print("",quote = FALSE)
      print(paste(oriCSVfile, "was converted at:", format(Sys.time(),"%Y-%B-%d %H:%M:%S")),quote = FALSE)
      print("",quote = FALSE)
      write.table(pkmat,file=saveCSVfile,col.names=TRUE,row.name=FALSE,quote=FALSE,sep=",")
  }   
      print("",quote = FALSE)
      print("Agilent data format conversion was done",quote = FALSE)
      print("",quote = FALSE)
}
